PROGRAM_NAME='Main, RMS 4_6, Base'
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 04/16/2020  AT: 10:53:29        *)
(***********************************************************)
#include 'SNAPI';
#include 'G4API';


(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

    dvMASTER 		= 0:1:0;

    dvIP_BSS_BLU	= 0:3:0;	// Blu50 Audio DSP at Student Station
    dvIP_ROKU		= 0:4:0;	// Roku STB at Student Station

    dvIP_LIGHTS		= 0:5:0;	// Loopback lighting device

    dvDVX_IR_APPLETV	= 5001:11:0;	// Apple TV IR

    dvDVX_REL		= 5001:21:0;	// Relays
					    // 1 = Screen Up
					    // 2 = Screen Down
					    // 3 = Lift Up
					    // 4 = Lift Down

    dvDVX_IO		= 5001:22:0;	// IOs	
					    // 1 = Shades Open
					    // 2 = Shades Close

    dvDVX_SWITCHER_IO_1	= 5002:1:0;	// / Amp out
    dvDVX_SWITCHER_IO_2	= 5002:2:0;	// DVD In (emulated source)
    dvDVX_SWITCHER_IO_3	= 5002:3:0;	// Apple TV In / Display Out (DXLink)
    dvDVX_SWITCHER_IN_4	= 5002:4:0;	// Roku In
    dvDVX_SWITCHER_IN_5	= 5002:5:0;	// Enzo In (DXLink)
    dvDVX_SWITCHER_IN_6	= 5002:6:0;	// Laptop In (DXLink)

    dvDXLINK_TX_1	= 5301:1:0;	// Enzo TX
    dvDXLINK_TX_2	= 5302:1:0;	// Laptop TX

    dvDXLINK_RX_1	= 5401:1:0;	// Display RX

    dvENZO		= 6001:1:0;

    dvRS232_DISPLAY	= 5001:2:0;	// Video Monitor	: Dummy loopback device
    dvRS232_PROJECTOR	= 5001:3:0;	// Video Projector 	: Dummy loopback device	
    dvRS232_CONFIDENCE	= 5001:4:0;     // Video Monitor -> Direct controlled serial device - no module code
    
    dvNAV_ENCODER_1	= 8001:1:0	// SVSI NMX-ENC-N2412A

    dvTP_1		= 10001:1:0;
    dvTP_1_ROKU		= 10001:2:0;
    dvTP_1_APPLE	= 10001:3:0;
    dvTP_1_LIGHTS	= 10001:4:0;
    dvTP_1_DSP		= 10001:5:0;
    dvTP_1_ENZO		= 10001:6:0;
    
    dvTP_1_RMS		= 10001:7:0;
    dvTP_1_MEET		= 10001:8:0;

    vdvLIGHTS		= 33001:1:0;
    vdvPROJECTOR	= 33002:1:0;

    vdvLaptop           = 33004:1:0;    // NL Virtual Device for Source Usage Tracking
    
    vdvBSS_BLU_1	= 41101:1:0;
    vdvBSS_BLU_2	= 41101:2:0;	// Room Out Volume
    vdvBSS_BLU_3	= 41101:3:0;	// Mic Mixer Input Gain 
    vdvBSS_BLU_4	= 41101:4:0;	// VTC Input Gain

    vdvROKU		= 41102:1:0;

    vdvDISPLAY		= 41103:1:0;

    vdvRMS		= 41001:1:0;
    vdvRMSGui		= 41002:1:0;

    vdvRMSSourceUsage   = 36001:1:0  // RMS Source Usage Monitor
     
(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT
    // Relay constants
    integer SCREEN_UP	 = 1;
    integer SCREEN_DOWN  = 2;
    integer LIFT_UP	 = 3;
    integer LIFT_DOWN	 = 4;
    // IO constants
    integer SHADES_OPEN	 = 1;
    integer SHADES_CLOSE = 2;
    
    integer DVX_STANDBY	 = 100;
    
#WARN 'SET DEVICE COMMUNICATION PROPERTIES AT CONSTANT DEFINITIONS'
    // Lights - Just a dummy, can be anything
    char IP_ADDRESS_LIGHTS[ ]	= '10.10.31.101';	
    // Roku - Set to the address of the device assigned to your station
    char IP_ADDRESS_ROKU[ ] 	= '192.168.2.106';	
    // BSS Blu50 - Set to the address of the device assigned to your station
    char IP_ADDRESS_DSP[ ]  	= '192.168.2.233';	
    char DEVICE_ID_DSP[ ] 	= '0x1';

    // Remove these devices from Auto Registration process
    dev RMS_FILTER_DEVICES[ ] = {5001:1:0};
    
(***********************************************************)
(*               TIMELINE DEFINITIONS GO BELOW             *)
(***********************************************************)
    long TL_SYSTEM_FB	 = 1;
    
(***********************************************************)
(* VARIABLE DEFINITIONS GO BELOW *)
(***********************************************************)
DEFINE_VARIABLE
volatile CHAR MyXDDFile[] = 'Samsung_Monitor_LightAmp_1.0.0.xdd'

// RMS Monitor parameters
VOLATILE CHAR LAPTOP_NAME[20] = 'Laptop';
VOLATILE CHAR LAPTOP_DESCRIPTION[50] = 'Laptop Input';

// RMS Modes
volatile char sSystemModes[ ]  = 'Conference|Presentation|Entertainment';

// RMS GUI
volatile dev dvTP_BASE[ ] = {dvTP_1};
volatile dev dvTP_RMS[ ] = {dvTP_1_RMS};

// RMS DevChan Motor Monitor parameters
volatile integer nScreenChannels[ ] = {1, 2};

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

([dvDVX_REL, SCREEN_UP], [dvDVX_REL, SCREEN_DOWN])
([dvDVX_REL, LIFT_UP], [dvDVX_REL, LIFT_DOWN])
([dvDVX_IO, SHADES_OPEN], [dvDVX_IO, SHADES_CLOSE])


#include 'Roku';
#include 'AppleTV';
#include 'Enzo';
#include 'BSSBlu50';
#include 'Lighting';
#include 'System';

#DEFINE INCLUDE_RMS_EVENT_SYSTEM_POWER_REQUEST_CALLBACK
DEFINE_FUNCTION RmsEventSystemPowerChangeRequest(CHAR powerOn)
{
    if(powerOn) 
    {
	SetSystemPower(true);
    }
    else
    {
	SetSystemPower(false);
    }
}

#DEFINE INCLUDE_RMS_EVENT_SYSTEM_MODE_REQUEST_CALLBACK
DEFINE_FUNCTION RmsEventSystemModeChangeRequest(CHAR newMode[])
{
    switch(newMode)
    {
	case 'Entertainment':
	{
	    setSystemInput(3); // Roku
	}
	case 'Conference':
	{
	    setSystemInput(5); // Enzo
	}
	case 'Presentation':
	{
	    setSystemInput(6); // Laptop
	}
    }
}

#include 'RmsEventListener';

#include 'RmsApi';
#INCLUDE 'RmsSourceUsage';

#include 'rmsMeeting';


(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START

RmsSourceUsageAssignAssetMutExcl(1, dvIP_ROKU);
RmsSourceUsageAssignAssetMutExcl(2, dvDVX_IR_APPLETV);
RmsSourceUsageAssignAssetMutExcl(3, dvENZO);
RmsSourceUsageAssignAssetMutExcl(4, vdvLaptop);

// reset all sources on program startup
RmsSourceUsageReset();


(***********************************************************)
(*               MODULE DEFINITIONS GO BELOW               *)
(***********************************************************)
DEFINE_MODULE
    'RmsNetLinxAdapter_dr4_0_0' RmsNetLinxAdapter_dr4_0_0_1 (vdvRMS);
    'RmsClientGui_dr4_0_0' 	mdlRMSGUI_1(vdvRMSGui,dvTp_RMS,dvTP_BASE);
     
    'RmsSystemPowerMonitor'     RmsSystemPowerMonitor_1(vdvRMS, dvMASTER);
    'RmsSystemModeMonitor'  	RmsSystemModeMonitor_1(vdvRMS, dvMASTER, sSystemModes);
  
    'RmsDuetSettopBoxMonitor' 	RmsDuetSettopBoxMonitor_1( vdvRMS, vdvROKU, dvIP_ROKU );
    'RmsDuetMonitorMonitor' 	RmsDuetMonitorMonitor_1( vdvRMS, vdvDISPLAY, dvRS232_DISPLAY );
//    'RmsDuetAudioMixerMonitor' 	RmsDuetAudioMixerMonitor_1( vdvRMS, vdvBSS_BLU_1, dvIP_BSS_BLU );
    'RmsDuetBssBlu50RPMMonitor' 	RmsDuetBssBlu50RPMMonitor_1( vdvRMS, vdvBSS_BLU_1, dvIP_BSS_BLU );
    
    'RmsNlLightSystemMonitor' 	RmsNlLightSystemMonitor_1( vdvRMS, vdvLIGHTS, dvIP_LIGHTS );
//    'RmsNlVideoProjectorMonitor'RmsNlVideoProjectorMonitor_1( vdvRMS, vdvPROJECTOR, dvRS232_PROJECTOR );
    'RmsNlScoLightAmpVideoProjectorMonitor' RmsNlScoLightAmpVideoProjectorMonitor_1( vdvRMS, vdvPROJECTOR, dvRS232_PROJECTOR );
    'RmsGenericNetLinxDeviceMonitor' RmsGenericNetLinxDeviceMonitor_1( vdvRMS, dvDVX_IR_APPLETV );
    'RmsVirtualDeviceMonitor' 	mdlRmsLaptopVdvMonitorMod_1(vdvRMS, vdvLaptop, LAPTOP_NAME, LAPTOP_DESCRIPTION);
   
    'BSS_BLU50_Comm_dr2_0_0' 	mdlBSS_BLU50_Comm_dr2_0_0_1(vdvBSS_BLU_1, dvIP_BSS_BLU);
    'Roku_Comm_dr1_0_0' 	mdlRoku_Comm_dr1_0_0_1(vdvROKU, dvIP_ROKU);
    'CommSCOLightAmp' 		mdlCommSCOLightAmp_1(vdvPROJECTOR, dvRS232_PROJECTOR);
    'CommZ10Lights' 		mdlCommZ10Lights_1(vdvLIGHTS, dvIP_LIGHTS);
    
    'DeviceDriverEngine' 	MyDevice_1(vdvDISPLAY, dvRS232_DISPLAY, MyXDDFile)

    'RmsDevChanDeviceMonitor'   RmsDevChanDeviceMonitor_1(vdvRMS, dvDVX_REL, nScreenChannels);
    
    'RmsDirectControlDeviceMonitor' RmsDirectControlDeviceMonitor_1(vdvRMS, dvRS232_CONFIDENCE);
    
//    'RmsGenericNetLinxDeviceMonitor' RmsGenericNetLinxDeviceMonitor_2( vdvRMS, dvNAV_ENCODER_1 ); <- DeviceID of SVSI Endpoint not in RMS database !!!
    'RmsSVSiDeviceMonitor' RmsSVSiDeviceMonitor_1( vdvRMS, dvNAV_ENCODER_1 ); 

(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT
data_event[vdvRMS]
{
    online:
    {
	// Configure Client
	send_command data.device,'CONFIG.CLIENT.ENABLED-true';
//	send_command data.device,"'CONFIG.CLIENT.NAME-',uRoomData.name";
//	send_command data.device,"'CONFIG.SERVER.URL-',uRmsData.url";
//	send_command data.device,"'CONFIG.SERVER.PASSWORD',uRmsData.password";
	
	// Confugure Auto Registration
	RmsSetDeviceAutoRegister(TRUE);
	RmsSetDeviceAutoRegisterFilter(RMS_FILTER_DEVICES);

	// Configure Asset Registrations
	RmsAssetControlMethodExclude(RmsDevToString(dvIP_ROKU), 'asset.power');
	send_command data.device,"'@',RmsDevToString(dvRS232_PROJECTOR),'-RMS-Asset-Name,My Projector'";
	send_command data.device,"'@',RmsDevToString(dvIP_LIGHTS),'-RMS-Asset-Name,My Lights'";
	send_command data.device,"'@',RmsDevToString(dvIP_LIGHTS),'-RMS-Asset-Make,Z-Lights'";
	send_command data.device,"'@',RmsDevToString(dvIP_LIGHTS),'-RMS-Asset-Model,Z-10'";
	send_command data.device,"'@',RmsDevToString(dvDVX_IR_APPLETV),'-RMS-Asset-Name,My AppleTV'";
	send_command data.device,"'@',RmsDevToString(dvDVX_IR_APPLETV),'-RMS-Asset-Make,Apple'";
	send_command data.device,"'@',RmsDevToString(dvDVX_IR_APPLETV),'-RMS-Asset-Model,AppleTV'";
	send_command data.device,"'@',RmsDevToString(dvDVX_IR_APPLETV),'-RMS-Asset-Type,',RMS_ASSET_TYPE_SETTOP_BOX";
	send_command data.device,"'@',RmsDevToString(vdvLaptop),'-RMS-Asset-Type,AUXILLIARY.INPUT'"; // Custom asset type

	send_command data.device,"'@',RmsDevToString(dvDVX_REL),'-RMS-Asset-Name,My Screen Motor'";
	send_command data.device,"'@',RmsDevToString(dvDVX_REL),'-RMS-Asset-Type,',RMS_ASSET_TYPE_MOTOR";
	
	send_command vdvRMS,"'@',RmsDevToString(dvRS232_CONFIDENCE),'-RMS-Asset-Name,Confidence Monitor'"
	send_command vdvRMS,"'@',RmsDevToString(dvRS232_CONFIDENCE),'-RMS-Asset-Make,Samsung'";
	send_command vdvRMS,"'@',RmsDevToString(dvRS232_CONFIDENCE),'-RMS-Asset-Model,SR24R'";
	send_command vdvRMS,"'@',RmsDevToString(dvRS232_CONFIDENCE),'-RMS-Asset-Type,',RMS_ASSET_TYPE_MONITOR";
    }
}

// NetLinx Module Device - force the asset registration
data_event[vdvPROJECTOR]
{
    online:
    {
	on[data.device, DATA_INITIALIZED];
    }
}

// Duet Module Devices
data_event[vdvROKU]
{
    online:
    {
	send_command data.device,'PROPERTY-RMS-Asset-Name,My Roku';
	send_command data.device,'PROPERTY-Has-Power,true';
	send_command data.device,'PROPERTY-Has-Station,false';
    }
}

data_event[vdvBSS_BLU_1]
{
    online:
    {
	send_command data.device,'PROPERTY-RMS-Asset-Name,My DSP';
    }
}

// XDD Module Device
data_event[vdvDISPLAY]
{
    online:
    {
	send_command data.device,'PROPERTY-RMS-Asset-Name,My Monitor';
    }
}




    
    
    

